/**
 *
 * Juego del puzzle (N^2 - 1)
 *
 * Para llamar al programa: n_puzzle d
 *
 * donde el tablero sera de dimensiones d x d,
 * y d debe estar entre [DIM_MIN,DIM_MAX]
 */

#include "utils.h"
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>

// constantes
#define DIM_MIN 3
#define DIM_MAX 9

// tablero
int tablero[DIM_MAX][DIM_MAX];

// dimensiones
int d;

// prototipos
void clear(void);
void greet(void);
void init(void);
void draw(void);
bool move(int);
bool won(void);

int main(int argc, string argv[])
{
    // Asegura uso apropiado
    if (argc != 2)
    {
        printf("Uso: n_puzzle d\n");
        return 1;
    }

    // Asegura dimensiones validas
    d = atoi(argv[1]);
    if (d < DIM_MIN || d > DIM_MAX)
    {
        printf("Las dimensiones del tablero deben estar entre %i x %i and %i x %i\n",
            DIM_MIN, DIM_MIN, DIM_MAX, DIM_MAX);
        return 2;
    }

    // Abrir log
    FILE* file = fopen("log.txt", "w");
    if (file == NULL)
    {
        return 3;
    }

    // Saludar al usuario
    greet();

    // Inicializar el tablero
    init();

    // pedir movidas hasta que se gane
    while (true)
    {
        // limpiar pantalla
        clear();

        // dibujar el estado actual del tablero
        draw();

        // guardar en un log el estado actual del tablero(para testear)
        for (int i = 0; i < d; i++)
        {
            for (int j = 0; j < d; j++)
            {
                fprintf(file, "%i", tablero[i][j]);
                if (j < d - 1)
                {
                    fprintf(file, "|");
                }
            }
            fprintf(file, "\n");
        }
        fflush(file);

        // checkear si se gano
        if (won())
        {
            printf("ftw!\n");
            break;
        }

        // pedir al usuario que ingrese una movida
        printf("Tile to move: ");
        int tile = leer_int();

        // Salir del juego si el usuario ingresa 0 (para testear)
        if (tile == 0)
        {
            break;
        }

        // guardar en log el movimiento (para testear)
        fprintf(file, "%i\n", tile);
        fflush(file);

        // Mover en caso de ser posible, mostrar que el movimiento es ilegal en caso contario
        if (!move(tile))
        {
            printf("\nMovida ilegal.\n");
            Sleep(500);
        }

        // Espera (con propositos de animacion del juego)
        Sleep(500);
    }

    // cerrar log
    fclose(file);

    // retorna 0 en caso de exito
    return 0;
}

/**
 * Limpia pantalla utilizando llamadas al sistema.
 * OJO SOLO FUNCIONA EN WINDOWS...
 */
void clear(void)
{
    system("cls");
}

/**
 * Saluda al jugador. SOLO FUNCIONA EN WINDOWS (usa Sleep
 * de windows.h)
 */
void greet(void)
{
    clear();
    printf("BIENVENIDO AL N-PUZZLE\n");
    Sleep(2000);
}

/**
 * Inicializa el tablero del juego con las piezas numeradas desde 1
 * hasta d*d - 1 (es decir, llena el arreglo bidimensional con los
 * valores, pero no imprime el arreglo en pantalla)
 */
void init(void)
{
    // TODO
}

/**
 * Imprime el tablero en su estado actual.
 */
void draw(void)
{
    // TODO
}

/**
 * Si el espacio libre esta al lado de la pieza ingresada en entrada
 * mueve la pieza hacia el espacio libe.
 * @tile: Pieza a mover
 * @returns: true si la pieza se puede mover, falso en caso contrario.
 */
bool move(int tile)
{
    // TODO
    return false;
}

/**
 * Retorna true si se gano el juego (todas las piezas en su lugar).
 * retorna false en caso contrario.
 */
bool won(void)
{
    // TODO
    return false;
}
