#ifndef _UTILS_H
#define _UTILS_H

#include <float.h>
#include <limits.h>
#include <stdbool.h>
#include <stdlib.h>


/*
 * Se define un tipo de datos para strings
 */

typedef char *string;


/*
 * Lee una linea de texto desde la entrada estandar y retorna el char
 * equivalente; si el texto ingresado no representa un caracter, se le
 * pide al usuario que vuelva a ingresar un caracter. Los espacios que
 * se antepongan o se pongan al final de lo ingresado seran ignorados.
 * Si por alguna razon la linea no se puede leer, la funcion retorna
 * CHAR_MAX
 */

char leer_char(void);


/*
 * Lee una linea de texto desde la entrada estandar y retorna un double
 * equivalente al texto ingresado; Si el texto ingresado no representa
 * un double, se le pide al usuario que reingrese un valor. Si se
 * anteponen, o se ponen espacios al final del numero ingresado, estos
 * seran ignorados. Esta funcion no es capaz de detectar overflow y
 * underflow. Si una linea no puede leerse, se retornara DBL_MAX.
 */

double leer_double(void);

/*
 * Lee una linea de texto desde la entrada estandar y retorna un float
 * equivalente al texto ingresado; Si el texto ingresado no representa
 * un flotante, se le pide al usuario que reingrese un valor. Si se
 * anteponen, o se ponen espacios al final del numero ingresado, estos
 * seran ignorados. Esta funcion no es capaz de detectar overflow y
 * underflow. Si una linea no puede leerse, se retornara FLT_MAX.
 */

float leer_float(void);


/*
 * Lee una linea desde la entrada estandar y la retorna como un
 * int en el rango: [-2^31 + 1, 2^31 - 2]; Si el texto no representa
 * un entero, se le pedira al usuario que reingrese un valor. Los espacios
 * al inicio y final del texto ingresado seran ignorados. Esta funcion no
 * detexta overflow. Si no se puede leer la linea de texto, se retornara INT_MAX
 */

int leer_int(void);

/*
 * Lee una linea desde la entrada estandar y la retorna como un long
 * long en el rango: [-2^63 + 1, 2^63 - 2]; Si el texto no representa
 * un long long, se le pedira al usuario que reingrese un valor. Los espacios
 * al inicio y final del texto ingresado seran ignorados. Esta funcion no
 * detecta overflow. Si no se puede leer la linea de texto, se retornara LLONG_MAX
 */

long long leer_long_long(void);

/* Lee una linea de texto desde la entrada estandar y la retorna
 * como un char *, eliminando los '\n' (es decir, si el usuario
 * ingresa solo '\n', se retornara "", no NULL). Retorna NULL en
 * caso de error o si no se pudo leer linea (EOF por ejemplo). Los
 * espacios al inicio y al final de la entrada no son ignorados.
 * El string leido se guarda en el heap (a traves de malloc), por
 * lo que la memoria debera ser liberada mediante free para evitar
 * fugas de memoria.
 */

string leer_string(void);

#endif


